#usr/bin/perl
#use regex;
#use strict;

# This script uses regular expressions to extract for each abstract, the associated MeSH terms
# Next, MeSH terms are mined for tissue-related terms, diseases, drugs and genes.

# Input text format: pubmed abstracts in the MEDLINE format
# The script uses the following files:
# gene_terms.txt
# RXNSandNAMES.txt
# DO_NamesAndSynonyms.txt
# DO_ref_to_MeSH.txt

 

############### Global Variables ###############
my$para; my%mesh_id_name=''; my%drug_name='';
my%mesh_names_match=''; my$tissue_count=0; my$disease_count=0; my$co_count=0; my$co_count2=0; my$co_count3=0;
my$mesh_count=0; my%dis_id_name=''; my%dis_name=''; my%gene_id_name=''; my%gene_name='';

################################################

#@ARGV == 1 or die "usage: Map_MeSH_tissue_disease.pl INPUT_TEXT_FILE\n";
#my$InFile = shift;



open (CYT,"hgnc_gene_names_and_synonyms.txt") or die "can't open the text file"; #opening the input text file 
while (my$b2=<CYT>){
	my@sp2=split("\t", $b2);
	my$mid2=$sp2[0]; #id
	my$symb=$sp2[1];# Approved Symbol
	my$nam2=$sp2[2]; #Approved Name
	my$syn=$sp2[4]; my@syns=split(",",$syn); 
	my$entrez=$sp2[5];
	chomp($mid2,$entrez, $nam2);
	$nam2=uc($nam2); 
	if($nam2){$gene_id_name{$entrez}=$nam2;}
	if($nam2){$gene_name{$nam2}=1;}
	foreach my$sy(@syns){ 
		$sy=~s/ //g;
		chomp($sy);
		$sy=uc($sy);
		$gene_name{$sy}=1; #print "$sy\n";
	}
}
close(CYT);
print "finished reading input gene list file...\n";
open (TIS,"RXN_DRUGS_shorter.txt") or die "can't open the text file"; #opening the input text file 
while (my$b=<TIS>){
	my@sp=split("\t", $b);
	my$gen=$sp[0]; 
	my$nam=$sp[1];
	chomp($gen, $nam);
	$gen=uc($gen); $nam=uc($nam);
	#if($nam){$mesh_id_name{$mid}=$nam;}
	if($nam){$drug_name{$nam}=1;}

}
close(TIS);
print "finished reading input drug list file...\n";
open (DISS,"DO_NamesAndSynonyms.txt") or die "can't open the text file"; #opening the input text file 
while (my$d=<DISS>){
	my@sp1=split("\t", $d);
	my$did=$sp1[0];
	my$dnam=$sp1[1];
	chomp($did, $dnam);
	$did=uc($did); $dnam=uc($dnam);
	$dis_id_name{$did}=$dnam;
	$dis_name{$dnam}=1;
}
close(DISS);

print "finished reading input disease list file...\n";

open (STAT, ">statistics.txt") or die "can't open stats file";
open (OUT, ">terms_per_abs.txt") or die "can't open output file";
open (OUT2, ">disease2drugs_per_abs.txt") or die "can't open output file";



#$/="\n\n";
open (TEXT,"pubmed_result_Oct.txt") || die "Unable to open $open because\n$!\n";
while (my$line=<TEXT>){  
	$para = "$para"."$line";
	if($line=~/SO  - / or $line=~/\n\n/ or $line=~/\r\n\r\n/ or $line=~/\r\r/){
		if ($para=~/MH  -/g){
		$stats = $stats+1;
		Info($para, %drug_name, %dis_name);
		if($stats%100000){} else{print "checked $stats abstracts...\n";}
		}
	$para='';
	}
}
print "finished reading all the abstracts...\n";

#$/="\n";
close TEXT; 


close (OUT); close(OUT2);


###############################################
##### getting statistics for the analysis #####
print STAT "number of reviewed abstracts:	$stats\n";
print STAT "number of abstracts with any MeSH:	$mesh_count\n";
print STAT "number of abstracts with tissue-MeSH:	$drug_count\n";
print STAT "number of abstracts with disease-MeSH:	$disease_count\n";
print STAT "number of abstracts with disease-MeSH:	$gene_count\n";
print STAT "number of abstracts with disease and drug:	$co_count\n";
print STAT "number of abstracts with disease and gene:	$co_count2\n";
print STAT "number of abstracts with gene and drug:	$co_count3\n";

###############################################
################# subroutines #################
###############################################

# this function gets all MeSH information and prints to output file
sub Info{ 

	my@lines=split("\n",$para); my@meshTerms=''; my$pubID=''; my$date=''; my$title=''; my$M='';
		my$temppara=$para;
		$temppara=~s/\n/ /g; $temppara=~s/\r/ /g;
		my@tit=split("TI  \- ",$temppara); my$tit2 = $tit[1]; my@tit3=split("[A-Z][A-Z]  - ", $tit2); $title=$tit3[0]; $title=~s/  / /g; #get title
			 $title=~s/\r\n/ /g;  $title=~s/\r/ /g; $title=~s/\t/ /g; while ($title=~/  /){$title=~s/  / /g;}
			 chomp($title); $title=~s/\r/ /g; $title=~s/\n/ /g;

	foreach my$row(@lines){ # extracting relevant info' for database (for evidence table)
		 
		if ($row=~/PMID- (\d+)/){ # get pubmed ID
			$pubID = $1; chomp($pubID);
		} 
		if ($row=~/DP  \- (\d{4})/){ # get publication Year
			$date=$1;  chomp($date);
		}
		#if ($row=~/TI  \- (.*)/){ # get publication Year

		#} 
		if ($row=~/MH  \- (.*)/){ # get MeSH terms
			$M=$1;  while($M=~/  /g){$M=~s/  / /g;} # get rid of extra spaces
			$M=uc($M); chomp($M); chomp($M); $M=~s/\r//g; $M=~s/\n//g;# make all uppercase and chomp
			push (@meshTerms,"$M | ");
		}
		if ($row=~/TA  \- (.*)/){$publication = $1; chomp($publication); $publication=~s/\r//g; $publication=~s/\n//g;} # get publication
		if ($row=~/PT  \- (.*)/) {  # get evidence class 
		
			# Checking which class of evidence the abstract belongs to.
			if ($1=~/Review/){$class = 'Review';}
			elsif ($1=~/Clinical Trial/i){$class= 'ClinicalTrial';}
			elsif ($1=~/Case Study/i){$class = 'CaseReport';}
			elsif($1=~/Case Reports/i){$class = 'CaseReport';}
			elsif($1=~/Case Report/i){$class = 'CaseReport';}
			elsif($1=~/Letter/i){$class = 'Letter';}
			elsif ($1=~/Comparative Study/i){$class = 'ComparativeStudy';}
			elsif ($1=~/Journal Article/i){$class = 'JournalArticle';} 
			else {$class = 'Undefined';}
		}		

	}

	if(scalar(@meshTerms)>1){
		$mesh_count++; # counting number of abstracts  with any MeSH terms
	
		print OUT "$pubID\t$date\t$title\t$publication\t$class\t@meshTerms\n";
		my$b_count=0; my@bmt=''; my$d_count=0; my@dmt=''; my$c_count=0; my@cyt='';
		foreach my$term(@meshTerms){
			my$ct='';
			chomp($term); $term=~s/ \|//;
			if($term=~/\//gi){
				my@s=split("\/",$term);
				$ct=$s[0];
			}
			else{ 
				$ct=$term;
			}
			while($ct=~/^ /){$ct=~s/^ //;}
			while($ct=~/ $/){$ct=~s/ $//;}
			if(defined $drug_name{$ct}){ 
				push(@bmt,"$term | ");
				$b_count++;
			}
			if($dis_name{$ct}){
				push(@dmt,"$term | ");
				$d_count++;
			}
			if(defined $gene_name{$ct}){ 
				push(@cyt,"$term | ");
				$c_count++;
			}
		}

		if($d_count or $b_count or $c_count){print OUT2 "$pubID\t$date\t$title\t$publication\t$class\t@bmt\t@dmt\t@cyt\n";}
		if($b_count>=1){$drug_count++;} # count number of abstracts with at least one drug mesh term
		if($d_count>=1){$disease_count++;} # count number of abstracts with at least one disease mesh term
		if($c_count>=1){$gene_count++;} # count number of abstracts with at least one gene mesh term
		if($d_count>=1 and $b_count>=1){$co_count++;} # count number of abstracts with disease and drug
		if($d_count>=1 and $c_count>=1){$co_count2++;} # count number of abstracts with disease and gene
		if($c_count>=1 and $b_count>=1){$co_count3++;} # count number of abstracts with drug and gene
	}
}
#####################################